use std::{error::Error, fs::read_to_string};
use chrono::{FixedOffset, NaiveDateTime, TimeZone};
use regex::Regex;
use crate::game::Game;

use super::names::normalize_name;

pub fn parse_file(file: &str) -> Result<Vec<Game>, Box<dyn Error>> {
    let file = read_to_string(file)?;
    let games = file.lines().map(parse_line);

    games.clone().for_each(|e| {
        if let Err(err) = e { eprintln!("{}",err) }
    });

    Ok(games.flatten().collect())
}

fn parse_line(line: &str) -> Result<Game,Box<dyn Error>>{
    let regex = Regex::new(r"(?m)^(\d+/\d+/\d+, \d+:\d+) - (.*): (.*)\+(.*) (\d)\s*[-:]\s*(\d) (.*)\+(.*?)(\s*\(?OT\)?)?\s*$")?;

    let Some(caps) = regex.captures(line) else {
        return Err(format!("Failed to match regex: {}",line).into());
    };

    let capture_groups: Vec<String> = caps
            .iter()
            .skip(1)
            .flatten()
            .map(|cap| cap.as_str().to_string())  
            .collect();

    let date_time = NaiveDateTime::parse_from_str(capture_groups.first().unwrap().trim(), "%m/%d/%y, %H:%M")?;
    let fixed_offset = FixedOffset::east_opt(2 * 3600).expect("Invalid timezone offset");


    let author = capture_groups.get(1).unwrap().trim();

    let t1_0 = capture_groups.get(2).unwrap().trim();
    let t1_1 = capture_groups.get(3).unwrap().trim();

    let cups_t1: i8 = capture_groups.get(4).unwrap().parse()?;
    let cups_t2: i8 = capture_groups.get(5).unwrap().parse()?;

    let t2_0 = capture_groups.get(6).unwrap().trim();
    let t2_1 = capture_groups.get(7).unwrap().trim();

    let is_ot = capture_groups.get(8).is_some();

    let norm_t1_0 = normalize_name(t1_0).ok_or_else(|| format!("Unknown name: {} on line {}",t1_0,line))?;
    let norm_t1_1 = normalize_name(t1_1).ok_or_else(|| format!("Unknown name: {} on line {}",t1_1,line))?;
    let norm_t2_0 = normalize_name(t2_0).ok_or_else(|| format!("Unknown name: {} on line {}",t2_0,line))?;
    let norm_t2_1 = normalize_name(t2_1).ok_or_else(|| format!("Unknown name: {} on line {}",t2_1,line))?;
    let norm_author = normalize_name(author).unwrap_or_else(|| "unknown".to_string());

    Ok(Game{
        added: fixed_offset.from_utc_datetime(&date_time),
        author: norm_author,
        t0p0: norm_t1_0,
        t0p1: norm_t1_1,
        t1p0: norm_t2_0,
        t1p1: norm_t2_1,
        score: cups_t1 - cups_t2,
        ot: is_ot,
    })
}

